==> RealiBoxTestBiz-0.6.6.tar.gz: RealiBoxTestBiz-0.6.6/realiboxtestbiz/JsonPath/Assertion.py <==
'''断言类
  结合JsonPath使用，将JsonPath自定义函数的结果保存到断言类
中作为属性。通过 get() 获取保存的值。通过断言类中的断言方法执行
断言操作。

  断言打通用例对象方案：
JsonPath调用断言
1、通过jsonpath提取后创建的断言对象， JsonPath 初始化参数改变为：
    增加一个默认None的参数，用于接收需要解析的json数据，Reali..Base中json_path
    方法第一个位置固定传入，用例对象。
2、JsonPath 获取用例对象后，在初始化时创建一个类JsonPathResult，类属性case_object用于保存用例对象，属性result用于保存结果列表，
    属性result_len保存数据长度， 属性out为空
3、JsonPath提取方法，操作将提取结果存放在 JsonPathResult.out 属性中。然后返回 JsonPathResult
4、装饰器pack_result 接收JsonPathResult后，将提取结果 .out 和 用例对象case_object 给Assertion进行初始化

用例中生成 断言类实现断言
1、Reali..Base 增加方法 assertion 默认向 Assertion 传递用例对象，并接收一个值用于断言


'''


def do_assert(func):
    func_name = func.__name__
    '''断言结束后将 JsonPath 匹配到的结果返回'''
    def inner(*args, **kwargs):
        # 获取断言信息类
        AssertionResult = func(*args, **kwargs)
        # get直接返回值
        if func_name == 'get':
            return AssertionResult.assert_value
        try:
            # 写入断言函数名称
            AssertionResult.assert_func = func_name
            # 读取用例对象
            case_object = AssertionResult.__case_object
            # 断言信息收集
            assertion = AssertionResult.__dict__.copy()
            for k, v in AssertionResult.__dict__.items():
                if k.startswith('__'):
                    assertion.pop(k)
            case_object.step_assertions_list.append(assertion)
            # 执行断言
            assert AssertionResult.result
            # 断言正常返回jsonpath提取的结果
            return AssertionResult.assert_value

        except AssertionError:
            raise

        except Exception:
            raise

    return inner


class AssertionBase:

    def __init__(self, case_obj, value):
        self.case_obj = case_obj  # 用例对象
        self.value = value  # 断言值
        # 创建一个类用于记录断言数据, 初始化属性case_object为用例对象
        self.AssertionResult = type('AssertionResult', (), {
            '__case_object': case_obj,
            'assert_value': self.value,  # 需要断言的值
            'assert_func': '',  # 断言函数
            'expression': None,  # 断言表达式
            'result': True  # 断言结果
        })

    @do_assert
    def get(self):
        '''返回 JsonPath 匹配的结果值'''
        return self.AssertionResult


class Assertion(AssertionBase):
    equal = '_assert_equal'
    len = '_assert_len'
    true = '_assert_true'
    false = '_assert_false'
    gt = '_assert_gt'
    gte = '_assert_gte'
    lt = '_assert_lt'
    lte = '_assert_lte'
    len_gt = '_assert_len_gt'
    len_gte = '_assert_len_gte'
    len_lte = '_assert_len_lte'
    Lambda = '_assert_lambda'

    def __init__(self, case_obj, value):
        super().__init__(case_obj, value)

    @do_assert
    def _assert_equal(self, excepted, **kwargs):
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (self.value == excepted)
        #  记录断言表达式
        self.AssertionResult.expression = f'{self.value} == {excepted}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_len(self, excepted: int, **kwargs):
        excepted = int(excepted)
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (len(self.value) == excepted)
        #  记录断言表达式
        self.AssertionResult.expression = f'{len(self.value)} == {excepted}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_true(self, **kwargs):
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = self.value
        #  记录断言表达式
        self.AssertionResult.expression = f'{self.value}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_false(self, **kwargs):
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = not self.value
        #  记录断言表达式
        self.AssertionResult.expression = f'not {self.value}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_gt(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''大于'''
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (self.value > excepted)
        #  记录断言表达式
        self.AssertionResult.expression = f'{self.value} > {excepted}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_gte(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''大于等于'''
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (self.value >= excepted)
        #  记录断言表达式
        self.AssertionResult.expression = f'{self.value} >= {excepted}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_lt(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''小于'''
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (self.value < excepted)
        #  记录断言表达式
        self.AssertionResult.expression = f'{self.value} < {excepted}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_lte(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''小于等于'''
        self.AssertionResult.result = (self.value <= excepted)
        self.AssertionResult.expression = f'{self.value} <= {excepted}'
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_len_gt(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''判断可迭代对象长度 大于'''
        self.AssertionResult.result = (len(self.value) > excepted)
        self.AssertionResult.expression = f'{len(self.value)} > {excepted}'
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_len_lt(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''判断可迭代对象长度 小于'''
        self.AssertionResult.result = (len(self.value) < excepted)
        self.AssertionResult.expression = f'{len(self.value)} < {excepted}'
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_len_gte(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''判断可迭代对象长度 大于等于'''
        self.AssertionResult.result = (len(self.value) >= excepted)
        self.AssertionResult.expression = f'{len(self.value)} >= {excepted}'
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_len_lte(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''判断可迭代对象长度 小于'''
        self.AssertionResult.result = (len(self.value) <= excepted)
        self.AssertionResult.expression = f'{len(self.value)} <= {excepted}'
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_lambda(self, x, **kwargs):
        '''x为lambda函数, 默认传值为self.value'''
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (x(self.value))
        #  记录断言表达式
        self.AssertionResult.expression = f'lambda函数表达式无法解析， 当前断言的值：{self.value}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

==> RealiBoxTestBiz-0.8.6.tar.gz: RealiBoxTestBiz-0.8.6/realiboxtestbiz/JsonPath/Assertion.py <==
'''断言类
  结合JsonPath使用，将JsonPath自定义函数的结果保存到断言类
中作为属性。通过 get() 获取保存的值。通过断言类中的断言方法执行
断言操作。

  断言打通用例对象方案：
JsonPath调用断言
1、通过jsonpath提取后创建的断言对象， JsonPath 初始化参数改变为：
    增加一个默认None的参数，用于接收需要解析的json数据，Reali..Base中json_path
    方法第一个位置固定传入，用例对象。
2、JsonPath 获取用例对象后，在初始化时创建一个类JsonPathResult，类属性case_object用于保存用例对象，属性result用于保存结果列表，
    属性result_len保存数据长度， 属性out为空
3、JsonPath提取方法，操作将提取结果存放在 JsonPathResult.out 属性中。然后返回 JsonPathResult
4、装饰器pack_result 接收JsonPathResult后，将提取结果 .out 和 用例对象case_object 给Assertion进行初始化

用例中生成 断言类实现断言
1、Reali..Base 增加方法 assertion 默认向 Assertion 传递用例对象，并接收一个值用于断言


'''


def do_assert(func):
    func_name = func.__name__
    '''断言结束后将 JsonPath 匹配到的结果返回'''
    def inner(*args, **kwargs):
        # 获取断言信息类
        AssertionResult = func(*args, **kwargs)
        # get直接返回值
        if func_name == 'get':
            return AssertionResult.assert_value
        try:
            # 写入断言函数名称
            AssertionResult.assert_func = func_name
            # 读取用例对象
            case_object = AssertionResult.__case_object
            # 断言信息收集
            assertion = AssertionResult.__dict__.copy()
            for k, v in AssertionResult.__dict__.items():
                if k.startswith('__'):
                    assertion.pop(k)
            case_object.step_assertions_list.append(assertion)
            # 执行断言
            assert AssertionResult.result
            # 断言正常返回jsonpath提取的结果
            return AssertionResult.assert_value

        except AssertionError:
            raise

        except Exception:
            raise

    return inner


class AssertionBase:

    def __init__(self, case_obj, value):
        self.case_obj = case_obj  # 用例对象
        self.value = value  # 断言值
        # 创建一个类用于记录断言数据, 初始化属性case_object为用例对象
        self.AssertionResult = type('AssertionResult', (), {
            '__case_object': case_obj,
            'assert_value': self.value,  # 需要断言的值
            'assert_func': '',  # 断言函数
            'expression': None,  # 断言表达式
            'result': True  # 断言结果
        })

    @do_assert
    def get(self):
        '''返回 JsonPath 匹配的结果值'''
        return self.AssertionResult


class Assertion(AssertionBase):
    equal = '_assert_equal'
    notequal = '_assert_notequal'
    len = '_assert_len'
    true = '_assert_true'
    false = '_assert_false'
    gt = '_assert_gt'
    gte = '_assert_gte'
    lt = '_assert_lt'
    lte = '_assert_lte'
    len_gt = '_assert_len_gt'
    len_gte = '_assert_len_gte'
    len_lte = '_assert_len_lte'
    Lambda = '_assert_lambda'

    def __init__(self, case_obj, value):
        super().__init__(case_obj, value)

    @do_assert
    def _assert_notequal(self, excepted, **kwargs):
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (self.value != excepted)
        #  记录断言表达式
        self.AssertionResult.expression = f'{self.value} != {excepted}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_equal(self, excepted, **kwargs):
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (self.value == excepted)
        #  记录断言表达式
        self.AssertionResult.expression = f'{self.value} == {excepted}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_len(self, excepted: int, **kwargs):
        excepted = int(excepted)
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (len(self.value) == excepted)
        #  记录断言表达式
        self.AssertionResult.expression = f'{len(self.value)} == {excepted}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_true(self, *args, **kwargs):
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = self.value
        #  记录断言表达式
        self.AssertionResult.expression = f'{self.value}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_false(self, *args, **kwargs):
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = not self.value
        #  记录断言表达式
        self.AssertionResult.expression = f'not {self.value}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_gt(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''大于'''
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (self.value > excepted)
        #  记录断言表达式
        self.AssertionResult.expression = f'{self.value} > {excepted}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_gte(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''大于等于'''
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (self.value >= excepted)
        #  记录断言表达式
        self.AssertionResult.expression = f'{self.value} >= {excepted}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_lt(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''小于'''
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (self.value < excepted)
        #  记录断言表达式
        self.AssertionResult.expression = f'{self.value} < {excepted}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_lte(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''小于等于'''
        self.AssertionResult.result = (self.value <= excepted)
        self.AssertionResult.expression = f'{self.value} <= {excepted}'
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_len_gt(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''判断可迭代对象长度 大于'''
        self.AssertionResult.result = (len(self.value) > excepted)
        self.AssertionResult.expression = f'{len(self.value)} > {excepted}'
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_len_lt(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''判断可迭代对象长度 小于'''
        self.AssertionResult.result = (len(self.value) < excepted)
        self.AssertionResult.expression = f'{len(self.value)} < {excepted}'
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_len_gte(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''判断可迭代对象长度 大于等于'''
        self.AssertionResult.result = (len(self.value) >= excepted)
        self.AssertionResult.expression = f'{len(self.value)} >= {excepted}'
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_len_lte(self, excepted: int, **kwargs):
        excepted = int(excepted)
        '''判断可迭代对象长度 小于'''
        self.AssertionResult.result = (len(self.value) <= excepted)
        self.AssertionResult.expression = f'{len(self.value)} <= {excepted}'
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult

    @do_assert
    def _assert_lambda(self, x, **kwargs):
        '''x为lambda函数, 默认传值为self.value'''
        # 断言方式实际是将断言的表达式， 和用户传递的数据传给 AssertionResult
        #  记录断言结果
        self.AssertionResult.result = (x(self.value))
        #  记录断言表达式
        self.AssertionResult.expression = f'lambda函数表达式无法解析， 当前断言的值：{self.value}'
        #  收集用户参数
        for k, v in kwargs.items():
            setattr(self.AssertionResult, k, v)
        return self.AssertionResult
